local metadata = {
"## Interface: FS15 1.1.0.0 RC12",
"## Title: Cargo",
"## Notes: Specialization fr Anhnger um Stckgut einzeln zu entladen",
"## Author: Marhu",
"## Version: 3.5.2",
"## Date: 27.11.2014",
"## Web: http://Marhu.net"
}

local FillTypeIsCargo ={}

Cargo = {};

function Cargo.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function Cargo:load(xmlFile)
	
	if g_currentMission.husbandries then
		for k,t in pairs(g_currentMission.husbandries) do
			local fillType = Fillable.fillTypeNameToInt[k]
			if fillType ~= nil then
				FillTypeIsCargo[fillType] = true;
			end
		end;
	end;
	
	local fillTypes = getXMLString(xmlFile, "vehicle.CargoFillTypes#fillTypes")
	if fillTypes ~= nil then
		local types = Utils.splitString(" ", fillTypes)
		for k,v in pairs(types) do
			local fillType = Fillable.fillTypeNameToInt[v]
			if fillType ~= nil then
				FillTypeIsCargo[fillType] = true
			 end
		end
	end
	
	self.updateTipping = Utils.overwrittenFunction(self.updateTipping, Cargo.updateTipping);
	
	self.CargoCapacity = {}
	self.CargoCapacity["default"] = self.capacity;
	local key0 = "vehicle.CargoCapacity.capacity(%d)"
	local num = 0;
	while hasXMLProperty(xmlFile, string.format(key0, num)) do
		local key1 = string.format(key0, num)
		local fillType = Fillable.fillTypeNameToInt[getXMLString(xmlFile, key1.."#fillType")]
		if fillType ~= nil then
			self.CargoCapacity[fillType] = Utils.getNoNil(getXMLFloat(xmlFile, key1.."#value"),0)
		end;
		num = num + 1;
	end
		
	self.CargoUnloadTimes = {}
	self.CargoUnloadTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.CargoUnloadTimes#value"), 1000);
	self.liveStock = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.liveStock#index"));
	self.baleGates = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.baleGates#index"));
	self.CargoUnloadTimes["default"] = self.CargoUnloadTime
	local key0 = "vehicle.CargoUnloadTimes.UnloadTime(%d)"
	local num = 0;
	while hasXMLProperty(xmlFile, string.format(key0, num)) do
		local key1 = string.format(key0, num)
		local fillType = Fillable.fillTypeNameToInt[getXMLString(xmlFile, key1.."#fillType")]
		if fillType ~= nil then
			self.CargoUnloadTimes[fillType] = Utils.getNoNil(getXMLFloat(xmlFile, key1.."#value"),0)
		end
		num = num + 1;
	end
		
	self.fillSounds = {};
	local fillSounds = getXMLString(xmlFile, "vehicle.fillSound#file")
    if fillSounds ~= nil and fillSounds ~= "" then
		self.fillSoundFile = fillSounds
		self.fillSounds["default"]={}
		self.fillSounds["default"].file = fillSounds
		self.fillSounds["default"].PitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillSounds#pitchOffset"), 1.0);
		self.fillSounds["default"].Volume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillSounds#volume"), 1.0);
		local key0 = "vehicle.fillSound.Sound(%d)"
		local num = 0;
		while hasXMLProperty(xmlFile, string.format(key0, num)) do
			local key1 = string.format(key0, num)
			local fillType = Fillable.fillTypeNameToInt[getXMLString(xmlFile, key1.."#fillType")]
			if fillType ~= nil then
				self.fillSounds[fillType] = {}
				self.fillSounds[fillType].file = getXMLString(xmlFile, key1.."#file")
				if self.fillSounds[fillType].file ~= nil and self.fillSounds[fillType].file ~= "" then
					self.fillSounds[fillType].PitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, key1.."#pitchOffset"), 1.0);
					self.fillSounds[fillType].Volume = Utils.getNoNil(getXMLFloat(xmlFile, key1.."#volume"), 1.0);
				else
					self.fillSounds[fillType] = nil
				end;
			end
			num = num + 1
		end
	end	
	
	for k,v in pairs(self.fillTypes) do
		if FillTypeIsCargo[k] then
			if not self.CargoCapacity[k] then
				self.CargoCapacity[k] = self.CargoCapacity["default"]
			end
			if not self.CargoUnloadTimes[k] then
				self.CargoUnloadTimes[k] = self.CargoUnloadTimes["default"]
			end
			if self.fillSounds["default"] and not self.fillSounds[k] then
				self.fillSounds[k] = self.fillSounds["default"]
			end
		end;
	end;
	setVisibility(self.liveStock, false);
	self.isSoundReinitDone = false;
	local pigS = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.pigS#index"));
	if pigS ~= nil then
		self.pigs = pigS;
	end;
	local beefS = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.beefS#index"));
	if beefS ~= nil then
		self.beef = beefS;
	end;
	local UnKnown = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.UnKnown#index"));
	if UnKnown ~= nil then
		self.UnKnown = UnKnown;
	end;
	local chickenS = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.chickenS#index"));
	if chickenS ~= nil then
		self.chicken2 = chickenS;
	end;
end;

function Cargo:delete()
	delete(self.sampleFillSound.sample);
	self.sampleFillSound.sample = nil;
end;

function Cargo:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Cargo:keyEvent(unicode, sym, modifier, isDown)
end;

function Cargo:update(dt)

	local reinitSound = false;
	if self.fillLevel > 0 and self.fillULevel == 0 then
		if self.isSoundReinitDone == false then
			self.capacity = self.CargoCapacity[self.currentFillType] or self.CargoCapacity["default"];
			self.CargoUnloadTime = self.CargoUnloadTimes[self.currentFillType] or self.CargoUnloadTimes["default"];
			self:setFillLevel(self:getFillLevel(self.currentFillType), self.currentFillType, false);
			reinitSound = true;
		end;
	else
		self.isSoundReinitDone = false;
	end;

	if reinitSound then
		if self.fillSoundFile then
			local soundFile = self.fillSounds[self.currentFillType] or self.fillSounds["default"]
			if self.fillSoundFile ~= soundFile and self.currentFillType ~= 0 then
				self.fillSoundFile = soundFile
				if self.fillSound ~= nil then
					delete(self.fillSound);
				end;
				local fillSound  = Utils.getFilename(self.fillSounds[self.currentFillType].file, self.baseDirectory);
				self.sampleFillSound.sample = createSample("fillSound");
				loadSample(self.sampleFillSound.sample, fillSound, false);
				self.sampleFillSound.pitchOffset = self.fillSounds[self.currentFillType].PitchOffset;
				setSamplePitch(self.sampleFillSound.sample, self.sampleFillSound.pitchOffset);
				self.sampleFillSound.duration = getSampleDuration(self.sampleFillSound.sample);
				self.sampleFillSound.volume = self.fillSounds[self.currentFillType].Volume;
				self.sampleFillSound.isPlaying = false;
			end;
		end;
		self.isSoundReinitDone = true;
	end;
	
	if self.attacherVehicle ~= nil then -- do nothing if trailer is orphan
		local volume = 0;
		if self:getIsActiveForInput(false) then
			volume = 1.5;
		elseif g_currentMission.controlledVehicle == nil then --player on foot
			--if g_currentMission.player ~= nil then
				local pvx, pvy, pvz = getWorldTranslation(g_currentMission.player.rootNode);
				local x, y, z = getWorldTranslation(self.rootNode);
				local farmerDistance = Utils.vector3Length(pvx-x, pvy-y, pvz-z);
				volume = (20-farmerDistance)/10
			--end;
		end;
		setSampleVolume(self.sampleFillSound.sample,volume*self.sampleFillSound.volume); 
		if self.tipState == 1 or self.tipState == 2 then
			if not self.sampleFillSound.isPlaying  then
				playSample(self.sampleFillSound.sample, 0, volume*self.sampleFillSound.volume, 0);
				self.sampleFillSound.isPlaying = true;
			end;
		else
			stopSample(self.sampleFillSound.sample);
			self.sampleFillSound.isPlaying = false;
		end;
	else
		stopSample(self.sampleFillSound.sample);
		self.sampleFillSound.isPlaying = false;
	end;
	if self.currentFillType ~= 0 then
	local desc = Fillable.fillTypeIndexToDesc[self.currentFillType];
	local name = desc.name;
		if name == 'pig' then
			setVisibility(self.pigs, true);
			setVisibility(self.beef, false);
			setVisibility(self.chicken2, false);
			setVisibility(self.UnKnown, false);	
			self.massScale = 0.0217;
		elseif name == 'beef' then
			setVisibility(self.pigs, false);
			setVisibility(self.beef, true);
			setVisibility(self.chicken2, false);
			setVisibility(self.UnKnown, false);
			self.massScale = 0.0228;
		elseif name == 'chicken2' then
			setVisibility(self.pigs, false);
			setVisibility(self.beef, false);
			setVisibility(self.chicken2, true);
			setVisibility(self.UnKnown, false);
			self.massScale = 0.0022;
		end;
	end;
	if self.currentFillType == 0 then
		setVisibility(self.pigs, false);
		setVisibility(self.beef, false);
		setVisibility(self.chicken2, false);
		setVisibility(self.UnKnown, true);
	end;
end;

function Cargo:updateTipping(superFunc, dt)
	if not self.isServer then
		return;
	end;
	if self:getCanTip() then
		
		local fillType = self.currentFillType;
		if FillTypeIsCargo[fillType] == nil then
			return superFunc(self,dt)
		end;
		local fillDelta = -self.fillLevel;
		if self.currentTipReferencePointIndex ~= nil then
			local animation = self.tipAnimations[self.currentTipReferencePointIndex];
			if animation ~= nil and animation.dischargeEndTime > animation.dischargeStartTime then
				if self:getCurrentTipAnimationTime() >= animation.dischargeStartTime then
					local m = self.capacity/((animation.dischargeEndTime-animation.dischargeStartTime)/animation.animationOpenSpeedScale);
					fillDelta = -m * dt;
				else
					fillDelta = 0;
				end
			end
		end
	   
		if fillDelta < 0 then
			fillDelta = -1
			self.LoadTime = (self.LoadTime or 0) + dt
			if self.LoadTime >= self.CargoUnloadTimes[fillType] then
				self.LoadTime = 0
				local curFill = self.fillLevel;
				self:setFillLevel(self.fillLevel + fillDelta, fillType);
				local fillDelta = self.fillLevel - curFill;
				self.lastFillDelta = fillDelta;
		   
				if self.currentTipTrigger ~= nil then
					if self.currentTipTrigger.updateTrailerTipping ~= nil then
						self.currentTipTrigger:updateTrailerTipping(self, fillDelta, fillType);
					end
				elseif self:getCanTipToGround() then
					if fillDelta < 0 then
						local x, _,z = getWorldTranslation(self.groundDropArea.start);
						local x1,_,z1 = getWorldTranslation(self.groundDropArea.width);
						local x2,_,z2 = getWorldTranslation(self.groundDropArea.height);
		   
						local area = {x,z,x1,z1,x2,z2, -fillDelta*self.fillLevelToTippedFillLevel};
						local area, bitType, valueAccumulation = TrailerTipAreaEvent.runLocally(area, fillType, self.groundAmountAccumulation)
						if area ~= nil then
							g_server:broadcastEvent(TrailerTipAreaEvent:new(area, bitType, fillType));
						end;
						if valueAccumulation ~= nil then
							self.groundAmountAccumulation = valueAccumulation;
						end;
					end;
				end;
			end;
		end;
	else
		self:onEndTip();
	end;
end

function Cargo:draw()
end;
